@@ -1,7 +1,9 @@
 import sys
+import urllib
 from base64 import b64decode, b64encode
 from collections import defaultdict
 from io import BytesIO
+from urllib.parse import urlencode
 
 __all__ = ("make_lambda_handler",)
 
@@ -68,7 +70,7 @@ def get_environ_v1(event, context, binary_support):
     environ = {
         "CONTENT_LENGTH": str(len(body)),
         "HTTP": "on",
-        "PATH_INFO": event["path"],
+        "PATH_INFO": urllib.parse.unquote(event["path"], encoding="iso-8859-1"),
         "REMOTE_ADDR": "127.0.0.1",
         "REQUEST_METHOD": event["httpMethod"],
         "SCRIPT_NAME": "",
@@ -87,14 +89,13 @@ def get_environ_v1(event, context, binary_support):
 
     # Multi-value query strings need explicit activation on ALB
     if "multiValueQueryStringParameters" in event:
-        # may be None when testing on console
-        multi_params = event["multiValueQueryStringParameters"] or {}
+        environ["QUERY_STRING"] = urlencode(
+            # may be None when testing on console
+            event["multiValueQueryStringParameters"] or (),
+            doseq=True,
+        )
     else:
-        single_params = event.get("queryStringParameters") or {}
-        multi_params = {key: [value] for key, value in single_params.items()}
-    environ["QUERY_STRING"] = "&".join(
-        "{}={}".format(key, val) for (key, vals) in multi_params.items() for val in vals
-    )
+        environ["QUERY_STRING"] = urlencode(event.get("queryStringParameters") or ())
 
     # Multi-value headers need explicit activation on ALB
     if "multiValueHeaders" in event:
@@ -134,11 +135,12 @@ def get_environ_v2(event, context, binary_support):
     body = get_body(event)
     headers = event["headers"]
     http = event["requestContext"]["http"]
+
     environ = {
         "CONTENT_LENGTH": str(len(body)),
         "HTTP": "on",
         "HTTP_COOKIE": ";".join(event.get("cookies", ())),
-        "PATH_INFO": http["path"],
+        "PATH_INFO": urllib.parse.unquote(http["path"], encoding="iso-8859-1"),
         "QUERY_STRING": event["rawQueryString"],
         "REMOTE_ADDR": http["sourceIp"],
         "REQUEST_METHOD": http["method"],
